//+--------------------------------------------------------------------------------------+
//|                                                             AdaptiveExtrapolator.mq4 |
//|                                                               Copyright  2009, gpwr |
//|                                                                                      |
//+--------------------------------------------------------------------------------------+
#property copyright "Copyright  2008, gpwr"
#property indicator_chart_window
#property indicator_buffers 6
#property indicator_color5 Blue
#property indicator_color6 Red
#property indicator_width5 2
#property indicator_width6 2


//Input parameters
extern string condition="nf>=lb + np>=lb";

extern int     lb  =30;   //   
/*extern int     lbstop   =100;
extern int     lbstep   =10;
*/
extern int     np =100;   // 
extern int     npstop=500;
extern int     npstep=5;

extern int     nf  =100;   //Number of bars to predict; for LP is set at PastBars-Order-1 
extern int     HarmNo   =20;    //Number of frequencies for Mathod 1; HarmNo=0 computes PastBars harmonics
extern double  FreqTOL  =0.0001;//Tolerance of frequency calculation for Method 1
//Indicator buffers
double pv[],pvreal[],pvmin[];
double fv[],fvreal[],fvmin[];

double realsko=10000.0;             // ,     

int lbreal,nfreal,lbmin,nfmin;      //   
//Global variables
int bars=0;

int init()
{   
   if(HarmNo==0) HarmNo=np;
   
   IndicatorBuffers(6);
   
   SetIndexBuffer(0,pv);
   SetIndexBuffer(1,fv);
   
   SetIndexBuffer(2,pvmin);
   SetIndexBuffer(3,fvmin);
   
   SetIndexBuffer(4,pvreal);
   SetIndexBuffer(5,fvreal);

   SetIndexShift(0,-lb);//past data vector 0..np-1; 0 corresponds to bar=lb
   SetIndexShift(1,nf-lb);//future data vector i=0..nf; nf corresponds to bar=lb
   SetIndexShift(2,-lb);
   SetIndexShift(3,nf-lb);
   SetIndexShift(4,-lb);
   SetIndexShift(5,nf-lb);

   SetIndexStyle(0,DRAW_LINE,STYLE_DOT,2);
   SetIndexStyle(1,DRAW_LINE,STYLE_DOT,2);
   SetIndexStyle(2,DRAW_LINE,STYLE_SOLID,2);
   SetIndexStyle(3,DRAW_LINE,STYLE_SOLID,2);
   SetIndexStyle(4,DRAW_LINE,STYLE_SOLID,2);
   SetIndexStyle(5,DRAW_LINE,STYLE_SOLID,2);
   
   return(0);
}

int deinit(){return(0);}

int start()
{



if (bars==Bars) return(0); //    
else
   {

while (np<npstop)
{
   
   ArrayInitialize(pv,EMPTY_VALUE);
   ArrayInitialize(fv,EMPTY_VALUE);  

   
//Find average of past values
   double av=0.0;
   for(int i=0;i<np;i++) av+=Open[i+lb];
   av/=np;
   
//Prepare data

      for(i=0;i<np;i++)
      {
         pv[i]=av;
         if(i<=nf) fv[i]=av;
      }

// 

         double w,m,c,s;
         for(int harm=0;harm<HarmNo;harm++)
         {
            Freq(w,m,c,s);
            for(i=0;i<np;i++) 
            {
               pv[i]+=m+c*MathCos(w*i)+s*MathSin(w*i);
               if(i<=nf) fv[i]+=m+c*MathCos(w*i)-s*MathSin(w*i);
            }         
         }
//Reorder the predicted vector
   for(i=0;i<=(nf-1)/2;i++)
   {
      double tmp=fv[i];
      fv[i]=fv[nf-i];
      fv[nf-i]=tmp;
   } 

//   ,  .   .    :   
  
   double ko,summko=0,sko=0;
   
   for (i=lb; i>=0; i--)
   {
      ko=MathPow((Close[i]-fv[nf-lb+i]),2);
      summko=ko+summko;
   }
   
   sko=MathSqrt(summko/(lb+1));

//  sko.    ,       -  .
   
   if (sko<realsko) 
   {
   realsko=sko;
   
   
   ArrayInitialize(pvmin,EMPTY_VALUE);
   ArrayInitialize(fvmin,EMPTY_VALUE);  

   ArrayCopy(fvmin,fv);
   ArrayCopy(pvmin,pv);
   }
   
//     .     .   
   
   np=np+npstep;
   }
   
   
   
   ArrayCopy(fvreal,fvmin);
   ArrayCopy(pvreal,pvmin);
   
   ArrayInitialize(pvmin,EMPTY_VALUE);
   ArrayInitialize(fvmin,EMPTY_VALUE);  
   ArrayInitialize(pv,EMPTY_VALUE);
   ArrayInitialize(fv,EMPTY_VALUE);  
 
   
   
   bars=Bars;
   }
   return(0); 

}



//+--------------------------------------------------------------------------------------+
//Quinn and Fernandes algorithm
void Freq(double& w, double& m, double& c, double& s)
{
   double z[],num,den;
   ArrayResize(z,np);
   double a=0.0;
   double b=2.0;
   z[0]=Open[lb]-pv[0];
   while(MathAbs(a-b)>FreqTOL)
   {
      a=b;
      z[1]=Open[1+lb]-pv[1]+a*z[0];
      num=z[0]*z[1];
      den=z[0]*z[0];
      for(int i=2;i<np;i++)
      {
         z[i]=Open[i+lb]-pv[i]+a*z[i-1]-z[i-2];
         num+=z[i-1]*(z[i]+z[i-2]);
         den+=z[i-1]*z[i-1];
      }
      b=num/den;
   }
   w=MathArccos(b/2.0);
   Fit(w,m,c,s);
   return;
}
//+-------------------------------------------------------------------------+
void Fit(double w, double& m, double& c, double& s)
{
   double Sc=0.0;
   double Ss=0.0;
   double Scc=0.0;
   double Sss=0.0;
   double Scs=0.0;
   double Sx=0.0;
   double Sxx=0.0;
   double Sxc=0.0;
   double Sxs=0.0;
   for(int i=0;i<np;i++)
   {
      double cos=MathCos(w*i);
      double sin=MathSin(w*i);
      Sc+=cos;
      Ss+=sin;
      Scc+=cos*cos;
      Sss+=sin*sin;
      Scs+=cos*sin;
      Sx+=(Open[i+lb]-pv[i]);
      Sxx+=MathPow(Open[i+lb]-pv[i],2);
      Sxc+=(Open[i+lb]-pv[i])*cos;
      Sxs+=(Open[i+lb]-pv[i])*sin;
   }
   Sc/=np;
   Ss/=np;
   Scc/=np;
   Sss/=np;
   Scs/=np;
   Sx/=np;
   Sxx/=np;
   Sxc/=np;
   Sxs/=np;
   if(w==0.0)
   {
      m=Sx;
      c=0.0;
      s=0.0;
   }
   else
   {
      //calculating c, s, and m
      double den=MathPow(Scs-Sc*Ss,2)-(Scc-Sc*Sc)*(Sss-Ss*Ss);
      c=((Sxs-Sx*Ss)*(Scs-Sc*Ss)-(Sxc-Sx*Sc)*(Sss-Ss*Ss))/den;
      s=((Sxc-Sx*Sc)*(Scs-Sc*Ss)-(Sxs-Sx*Ss)*(Scc-Sc*Sc))/den;
      m=Sx-c*Sc-s*Ss;
   }
   return;
}

